/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizconfig.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizconfig.entity.BizConfig;
import vip.xiaonuo.biz.modular.bizconfig.mapper.BizConfigMapper;
import vip.xiaonuo.biz.modular.bizconfig.param.BizConfigAddParam;
import vip.xiaonuo.biz.modular.bizconfig.param.BizConfigEditParam;
import vip.xiaonuo.biz.modular.bizconfig.param.BizConfigIdParam;
import vip.xiaonuo.biz.modular.bizconfig.param.BizConfigPageParam;
import vip.xiaonuo.biz.modular.bizconfig.service.BizConfigService;

import java.util.List;

/**
 * 系统配置Service接口实现类
 *
 * @author fanzherong
 * @date  2025/03/20 15:40
 **/
@Service
public class BizConfigServiceImpl extends ServiceImpl<BizConfigMapper, BizConfig> implements BizConfigService {

    @Override
    public Page<BizConfig> page(BizConfigPageParam bizConfigPageParam) {
        QueryWrapper<BizConfig> queryWrapper = new QueryWrapper<BizConfig>().checkSqlInjection();
        if(ObjectUtil.isAllNotEmpty(bizConfigPageParam.getSortField(), bizConfigPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(bizConfigPageParam.getSortOrder());
            queryWrapper.orderBy(true, bizConfigPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(bizConfigPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(BizConfig::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizConfigAddParam bizConfigAddParam) {
        long count = this.count();
        if(count>0){
            throw new CommonException("已添加过配置信息！");
        }
        BizConfig bizConfig = BeanUtil.toBean(bizConfigAddParam, BizConfig.class);
        this.save(bizConfig);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizConfigEditParam bizConfigEditParam) {
        BizConfig bizConfig = this.queryEntity(bizConfigEditParam.getId());
        BeanUtil.copyProperties(bizConfigEditParam, bizConfig);
        this.updateById(bizConfig);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizConfigIdParam> bizConfigIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizConfigIdParamList, BizConfigIdParam::getId));
    }

    @Override
    public BizConfig detail(BizConfigIdParam bizConfigIdParam) {
        return this.queryEntity(bizConfigIdParam.getId());
    }

    @Override
    public BizConfig queryEntity(String id) {
        BizConfig bizConfig = this.getById(id);
        if(ObjectUtil.isEmpty(bizConfig)) {
            throw new CommonException("系统配置不存在，id值为：{}", id);
        }
        return bizConfig;
    }
}
