/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizorder.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fhs.core.trans.anno.Trans;
import com.fhs.core.trans.constant.TransType;
import com.fhs.core.trans.vo.TransPojo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.common.pojo.CommonEntity;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 订单管理实体
 *
 * @author fanzherong
 * @date  2025/03/21 17:16
 **/
@Getter
@Setter
@TableName("biz_order")
public class BizOrder extends CommonEntity{

    /** 主键ID */
    @TableId
    @Schema(description = "主键ID")
    private String id;

    /** 订单编号 */
    @Schema(description = "订单编号")
    private String orderNumber;

    /** 订单名称 */
    @Schema(description = "订单名称")
    private String orderName;

    /** 客户id */
    @Schema(description = "客户id")
    private String customerId;

    /** 货品id */
    @Schema(description = "货品id")
    private String goodId;

    /** 订单类型（1:采购   2：销售） */
    @Trans(type = TransType.DICTIONARY, key = "order_type")
    private String orderType;

    /** 订单来源（1：oa推送   2：手动新增） */
    @Trans(type = TransType.DICTIONARY, key = "order_source")
    private String orderSource;

    /** 订单状态（1：正常  2：终止） */
    @Trans(type = TransType.DICTIONARY, key = "order_status")
    private String orderStatus;

    /** 订单重量 */
    @Schema(description = "订单重量")
    private BigDecimal orderWeight;

    /** 过磅重量 */
    @Schema(description = "过磅重量")
    private BigDecimal netWeight;

    /** 订单备注*/
    @Schema(description = "订单备注")
    private String orderRemark;


    @TableField(exist = false)
    /**客户名称*/
    private String customerName;

    @TableField(exist = false)
    /**货品名称*/
    private String goodsName;

    /**货品规格*/
    @TableField(exist = false)
    private String goodsModel;

    /** 联系人 */
    @TableField(exist = false)
    @Schema(description = "联系人")
    private String customerContactName;

    /** 手机号 */
    @TableField(exist = false)
    @Schema(description = "手机号")
    private String customerPhone;

    /** 客户地址 */
    @TableField(exist = false)
    @Schema(description = "客户地址")
    private String customerAddress;

    /**提货时间*/
    private String deliveryTimeId;

    /**提货开始时间*/
    @TableField(exist = false)
    private Date confStartTime;

    /**提货结束时间*/
    @TableField(exist = false)
    private Date confEndTime;

    /**签名*/
    private String orderSign;

    /**销售订单*/
    private String saleOrderInfo;

    /**审核备注*/
    private String orderReason;

    /**结束备注**/
    private String endReason;

    /**实际托运重量*/
    @TableField(exist = false)
    private BigDecimal actualWeight;

    /**卸货重量*/
    private BigDecimal shippingWeight;

}
