/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizorder.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import jakarta.servlet.http.HttpServletResponse;
import vip.xiaonuo.biz.modular.bizappointmentrecord.param.BizAppointmentRecordPageParam;
import vip.xiaonuo.biz.modular.customer.entity.BizCustomer;
import vip.xiaonuo.biz.modular.bizorder.entity.BizOrder;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderAddParam;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderEditParam;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderIdParam;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderPageParam;

import java.io.IOException;
import java.util.List;

/**
 * 订单管理Service接口
 *
 * @author fanzherong
 * @date  2025/03/21 17:16
 **/
public interface BizOrderService extends IService<BizOrder> {

    /**
     * 获取订单管理分页
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     */
    Page<BizOrder> page(BizOrderPageParam bizOrderPageParam);

    /**
     * 添加订单管理
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     */
    void add(BizOrderAddParam bizOrderAddParam);

    /**
     * 编辑订单管理
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     */
    void edit(BizOrderEditParam bizOrderEditParam);

    /**
     * 删除订单管理
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     */
    void delete(List<BizOrderIdParam> bizOrderIdParamList);

    /**
     * 获取订单管理详情
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     */
    BizOrder detail(BizOrderIdParam bizOrderIdParam);

    /**
     * 获取订单管理详情
     *
     * @author fanzherong
     * @date  2025/03/21 17:16
     **/
    BizOrder queryEntity(String id);

    /**获取客户id*/
    List<BizCustomer> getCustomerById(BizOrderIdParam bizOrderIdParam);

    /**绑定客户id*/
    void bindCustomerId(BizOrderEditParam bizOrderEditParam);


    /**导出报表*/
    void exportRecord(BizOrderPageParam bizOrderPageParam, HttpServletResponse response) throws IOException;

    /**结束订单*/
    void endOrder(BizOrderIdParam bizOrderIdParam);

    /**查询订单列表*/
    List<BizOrder> getList();

    /***订单签名*/
    void updateOrderSign(BizOrderEditParam bizOrderEditParam);

    /**确认*/
    void orderConfirm(BizOrderEditParam bizOrderEditParam);

    /**结束订单*/
    void endOrder(BizOrderEditParam bizOrderEditParam);

    /**审核**/
    void auditOrder(BizOrderEditParam bizOrderEditParam);

    /**提交*/
    void submit(BizOrderEditParam bizOrderEditParam);

    Page<BizOrder> getOrderTotal(BizOrderPageParam bizOrderPageParam);

    /**导出报表*/
    void exportRecordTotal(BizOrderPageParam bizOrderPageParam, HttpServletResponse response) throws IOException;
}
