/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.officialinfo.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.officialinfo.entity.OfficialInfo;
import vip.xiaonuo.biz.modular.officialinfo.param.OfficialInfoAddParam;
import vip.xiaonuo.biz.modular.officialinfo.param.OfficialInfoEditParam;
import vip.xiaonuo.biz.modular.officialinfo.param.OfficialInfoIdParam;
import vip.xiaonuo.biz.modular.officialinfo.param.OfficialInfoPageParam;
import vip.xiaonuo.biz.modular.officialinfo.service.OfficialInfoService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 微信公众号关注表控制器
 *
 * @author fanzherong
 * @date  2025/04/10 17:31
 */
@Tag(name = "微信公众号关注表控制器")
@RestController
@Validated
public class OfficialInfoController {

    @Resource
    private OfficialInfoService officialInfoService;

    /**
     * 获取微信公众号关注表分页
     *
     * @author fanzherong
     * @date  2025/04/10 17:31
     */
    @Operation(summary = "获取微信公众号关注表分页")
    @SaCheckPermission("/biz/officialinfo/page")
    @GetMapping("/biz/officialinfo/page")
    public CommonResult<Page<OfficialInfo>> page(OfficialInfoPageParam officialInfoPageParam) {
        return CommonResult.data(officialInfoService.page(officialInfoPageParam));
    }

    /**
     * 添加微信公众号关注表
     *
     * @author fanzherong
     * @date  2025/04/10 17:31
     */
    @Operation(summary = "添加微信公众号关注表")
    @CommonLog("添加微信公众号关注表")
    @SaCheckPermission("/biz/officialinfo/add")
    @PostMapping("/biz/officialinfo/add")
    public CommonResult<String> add(@RequestBody @Valid OfficialInfoAddParam officialInfoAddParam) {
        officialInfoService.add(officialInfoAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑微信公众号关注表
     *
     * @author fanzherong
     * @date  2025/04/10 17:31
     */
    @Operation(summary = "编辑微信公众号关注表")
    @CommonLog("编辑微信公众号关注表")
    @SaCheckPermission("/biz/officialinfo/edit")
    @PostMapping("/biz/officialinfo/edit")
    public CommonResult<String> edit(@RequestBody @Valid OfficialInfoEditParam officialInfoEditParam) {
        officialInfoService.edit(officialInfoEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除微信公众号关注表
     *
     * @author fanzherong
     * @date  2025/04/10 17:31
     */
    @Operation(summary = "删除微信公众号关注表")
    @CommonLog("删除微信公众号关注表")
    @SaCheckPermission("/biz/officialinfo/delete")
    @PostMapping("/biz/officialinfo/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<OfficialInfoIdParam> officialInfoIdParamList) {
        officialInfoService.delete(officialInfoIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取微信公众号关注表详情
     *
     * @author fanzherong
     * @date  2025/04/10 17:31
     */
    @Operation(summary = "获取微信公众号关注表详情")
    @SaCheckPermission("/biz/officialinfo/detail")
    @GetMapping("/biz/officialinfo/detail")
    public CommonResult<OfficialInfo> detail(@Valid OfficialInfoIdParam officialInfoIdParam) {
        return CommonResult.data(officialInfoService.detail(officialInfoIdParam));
    }
}
